@@ -1,7 +1,3 @@
-#ifndef SQUID_CONFIG_H
-#include "config.h"
-#endif
-
 #ifndef _SQUID_COMPAT_SHARED_H
 #define _SQUID_COMPAT_SHARED_H
 
@@ -17,17 +13,14 @@
  * of the requirements for wrapping your hack for safe portability.
  */
 
+#ifdef __cplusplus
 /*
  * Define an error display handler override.
  * If error_notify is set by the linked program it will be used by the local
  * portability functions. Otherwise perror() will be used.
  */
-#ifdef __cplusplus
-extern "C"
-#else
-extern
+extern void (*failure_notify) (const char *);
 #endif
-void (*failure_notify) (const char *);
 
 /*
  * sys/resource.h and sys/time.h are apparently order-dependant.
@@ -174,6 +167,9 @@ max(A const & lhs, A const & rhs)
  * Signalling flags are apparently not always provided.
  * TODO find out if these can be moved into specific OS portability files.
  */
+#if HAVE_SIGNAL_H
+#include <signal.h>
+#endif
 #ifndef SA_RESTART
 #define SA_RESTART 0
 #endif
@@ -212,6 +208,45 @@ extern "C" {
 #include "compat/xstrerror.h"
 #include "compat/xstring.h"
 #include "compat/xstrto.h"
+#include "compat/xis.h"
 
+#if !HAVE_MEMCPY
+#if HAVE_BCOPY
+#define memcpy(d,s,n) bcopy((s),(d),(n))
+#elif HAVE_MEMMOVE
+#define memcpy(d,s,n) memmove((d),(s),(n))
+#endif
+#endif
+
+#if !HAVE_MEMMOVE && HAVE_BCOPY
+#define memmove(d,s,n) bcopy((s),(d),(n))
+#endif
+
+/*
+ * strnstr() is needed. The OS may not provide a working copy.
+ */
+#if HAVE_STRNSTR
+/* If strnstr exists and is usable we do so. */
+#define squid_strnstr(a,b,c)    strnstr(a,b,c)
+#else
+/* If not we have our own copy imported from FreeBSD */
+const char * squid_strnstr(const char *s, const char *find, size_t slen);
+#endif
+
+#if __GNUC__
+#if !defined(PRINTF_FORMAT_ARG1)
+#define PRINTF_FORMAT_ARG1 __attribute__ ((format (printf, 1, 2)))
+#endif
+#if !defined(PRINTF_FORMAT_ARG2)
+#define PRINTF_FORMAT_ARG2 __attribute__ ((format (printf, 2, 3)))
+#endif
+#if !defined(PRINTF_FORMAT_ARG3)
+#define PRINTF_FORMAT_ARG3 __attribute__ ((format (printf, 3, 4)))
+#endif
+#else /* !__GNU__ */
+#define PRINTF_FORMAT_ARG1
+#define PRINTF_FORMAT_ARG2
+#define PRINTF_FORMAT_ARG3
+#endif
 
 #endif /* _SQUID_COMPAT_SHARED_H */
