@@ -51,7 +51,7 @@ public:
                                        HttpRequest * orig_request,
                                        StoreEntry * entry,
                                        HttpHeader * hdr_out,
-                                       http_state_flags flags);
+                                       const http_state_flags flags);
 
     virtual int dataDescriptor() const;
     /* should be private */
@@ -70,7 +70,7 @@ public:
     http_state_flags flags;
     size_t read_sz;
     int header_bytes_read;	// to find end of response,
-    int reply_bytes_read;	// without relying on StoreEntry
+    int64_t reply_bytes_read;	// without relying on StoreEntry
     int body_bytes_truncated; // positive when we read more than we wanted
     MemBuf *readBuf;
     bool ignoreCacheControl;
@@ -81,6 +81,10 @@ public:
 protected:
     virtual HttpRequest *originalRequest();
 
+    void processReply();
+    void proceedAfter1xx();
+    void handle1xx(HttpReply *msg);
+
 private:
     AsyncCall::Pointer closeHandler;
     enum ConnectionStatus {
@@ -97,6 +101,7 @@ private:
     void truncateVirginBody();
 
     virtual void haveParsedReplyHeaders();
+    virtual bool getMoreRequestBody(MemBuf &buf);
     virtual void closeServer(); // end communication with the server
     virtual bool doneWithServer() const; // did we end communication?
     virtual void abortTransaction(const char *reason); // abnormal termination
@@ -107,18 +112,20 @@ private:
 
     void writeReplyBody();
     bool decodeAndWriteReplyBody();
+    bool finishingBrokenPost();
+    bool finishingChunkedRequest();
     void doneSendingRequestBody();
     void requestBodyHandler(MemBuf &);
     virtual void sentRequestBody(const CommIoCbParams &io);
-    void sendComplete(const CommIoCbParams &io);
+    void wroteLast(const CommIoCbParams &io);
+    void sendComplete();
     void httpStateConnClosed(const CommCloseCbParams &params);
     void httpTimeout(const CommTimeoutCbParams &params);
 
     mb_size_t buildRequestPrefix(HttpRequest * request,
                                  HttpRequest * orig_request,
                                  StoreEntry * entry,
-                                 MemBuf * mb,
-                                 http_state_flags flags);
+                                 MemBuf * mb);
     static bool decideIfWeDoRanges (HttpRequest * orig_request);
     bool peerSupportsConnectionPinning() const;
 
