@@ -47,7 +47,7 @@ protected:
 };
 
 #if HAVE_AUTH_MODULE_BASIC
-#include "auth/basic/basicUserRequest.h"
+#include "auth/basic/UserRequest.h"
 class testAuthBasicUserRequest : public CPPUNIT_NS::TestFixture
 {
     CPPUNIT_TEST_SUITE( testAuthBasicUserRequest );
@@ -64,7 +64,7 @@ protected:
 #endif
 
 #if HAVE_AUTH_MODULE_DIGEST
-#include "auth/digest/digestUserRequest.h"
+#include "auth/digest/UserRequest.h"
 class testAuthDigestUserRequest : public CPPUNIT_NS::TestFixture
 {
     CPPUNIT_TEST_SUITE( testAuthDigestUserRequest );
@@ -81,7 +81,7 @@ protected:
 #endif
 
 #if HAVE_AUTH_MODULE_NTLM
-#include "auth/ntlm/ntlmUserRequest.h"
+#include "auth/ntlm/UserRequest.h"
 class testAuthNTLMUserRequest : public CPPUNIT_NS::TestFixture
 {
     CPPUNIT_TEST_SUITE( testAuthNTLMUserRequest );
@@ -98,7 +98,7 @@ protected:
 #endif
 
 #if HAVE_AUTH_MODULE_NEGOTIATE
-#include "auth/negotiate/negotiateUserRequest.h"
+#include "auth/negotiate/UserRequest.h"
 class testAuthNegotiateUserRequest : public CPPUNIT_NS::TestFixture
 {
     CPPUNIT_TEST_SUITE( testAuthNegotiateUserRequest );
