@@ -397,9 +397,6 @@ death(int sig)
         puts(dead_msg());
     }
 
-    if (shutting_down)
-        exit(1);
-
     abort();
 }
 
@@ -595,6 +592,12 @@ sig_child(int sig)
 #endif
 }
 
+void
+sig_shutdown(int sig)
+{
+    shutting_down = 1;
+}
+
 const char *
 getMyHostname(void)
 {
